package com.cultural.heritage.service.good.impl;

import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUnit;
import cn.hutool.core.date.DateUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.cultural.heritage.common.ErrorCode;
import com.cultural.heritage.constant.CommonConstant;
import com.cultural.heritage.constant.MqConstant;
import com.cultural.heritage.exception.BusinessException;
import com.cultural.heritage.mapper.CouponMapper;
import com.cultural.heritage.model.dto.coupon.CouponQueryRequest;
import com.cultural.heritage.model.entity.Coupon;
import com.cultural.heritage.model.entity.UserCoupon;
import com.cultural.heritage.service.good.CouponService;
import com.cultural.heritage.utils.MultiDelayMessage;
import com.cultural.heritage.utils.SqlUtils;
import com.cultural.heritage.utils.TimeSplitterUtils;
import jakarta.annotation.Resource;
import org.apache.commons.lang3.ObjectUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.util.Date;
import java.util.List;

@Service
public class CouponServiceImpl extends ServiceImpl<CouponMapper, Coupon> implements CouponService {


    @Resource
    private RabbitTemplate rabbitTemplate;


    @Override
    public QueryWrapper<Coupon> getQueryWrapper(CouponQueryRequest couponQueryRequest) {
        if (couponQueryRequest == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        Long id = couponQueryRequest.getId();
        String name = couponQueryRequest.getName();
        Date startTime = couponQueryRequest.getStartTime();
        Date endTime = couponQueryRequest.getEndTime();
        String sortField = couponQueryRequest.getSortField();
        String sortOrder = couponQueryRequest.getSortOrder();
        QueryWrapper<Coupon> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq(ObjectUtils.isNotEmpty(id), "id", id);
        queryWrapper.like(StringUtils.isNotBlank(name), "name", name);
        queryWrapper.ge(ObjectUtils.isNotEmpty(startTime), "startTime", startTime);
        queryWrapper.le(ObjectUtils.isNotEmpty(endTime), "endTime", endTime);
        queryWrapper.orderBy(SqlUtils.validSortField(sortField), sortOrder.equals(CommonConstant.SORT_ORDER_ASC),
                sortField);
        return queryWrapper;
    }


    @Override
    public Integer verifyIsReachTheUpperLimit(Long userId, Long couponId) {
        return this.baseMapper.verifyIsReachTheUpperLimit(userId, couponId);
    }


    /**
     * 校验
     */
    @Override
    public void validCoupon(Coupon coupon, boolean update) {
        Long id = coupon.getId();
        String name = coupon.getName();
        BigDecimal conditionAmount = coupon.getConditionAmount();
        BigDecimal standardAmount = coupon.getStandardAmount();
        Integer requirePoints = coupon.getRequirePoints();
        String startTime = coupon.getStartTime();
        String endTime = coupon.getEndTime();
        String status = coupon.getStatus();

        if (update) {
            if (id == null) {
                throw new BusinessException(ErrorCode.PARAMS_ERROR, "参数id错误");
            }
        }
        if (ObjectUtils.isEmpty(standardAmount) || standardAmount.compareTo(BigDecimal.ZERO) <= 0) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR, "标准价格错误");
        }
        if (ObjectUtils.isEmpty(conditionAmount) || conditionAmount.compareTo(BigDecimal.ZERO) <= 0) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR, "满减价格错误");
        }
        if (conditionAmount.compareTo(standardAmount) > 0) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR, "满减价格不能超过标准价格");
        }

        if (ObjectUtils.isEmpty(requirePoints) || requirePoints < 0) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR, "积分参数错误");
        }
        if (StringUtils.isAnyBlank(name, startTime, endTime, status)) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR, "存在参数为空");
        }
        if (startTime.compareTo(endTime) >= 0) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR, "有效期开始时间晚于有效期截止时间");
        }
    }



    /**
     * 向消息队列中发送优惠券创建的消息
     */
    @Override
    public void sendCouponCreateMessage(Coupon coupon) {
        //计算优惠券截止日期和当前的时间差
        DateTime now = new DateTime();
        DateTime end = DateUtil.parse(coupon.getEndTime());
        long seconds = DateUtil.between(now, end, DateUnit.SECOND);
        List<Long> splitTimeList = TimeSplitterUtils.splitTime(seconds);
        // 延迟检查优惠券是否过期
        MultiDelayMessage<Coupon> msg = new MultiDelayMessage<>(coupon, splitTimeList);
        int delayValue = msg.removeNextDelay().intValue();
        rabbitTemplate.convertAndSend(MqConstant.DELAY_EXCHANGE,
                MqConstant.DELAY_COUPON_ROUTING_KEY, msg, message -> {
                    // 添加延迟消息属性
                    message.getMessageProperties().setDelay(delayValue * 1000);
                    return message;
                });
    }




    /**
     * 向消息队列中发送用户优惠券创建的消息
     */
    @Override
    public void sendUserCouponCreateMessage(UserCoupon userCoupon, Coupon coupon) {
        //计算优惠券截止日期和当前的时间差
        DateTime now = new DateTime();
        DateTime end = DateUtil.parse(coupon.getEndTime());
        long seconds = DateUtil.between(now, end, DateUnit.SECOND);
        List<Long> splitTimeList = TimeSplitterUtils.splitTime(seconds);
        // 延迟检查优惠券是否过期
        MultiDelayMessage<Long> msg = new MultiDelayMessage<>(userCoupon.getId(), splitTimeList);
        int delayValue = msg.removeNextDelay().intValue();
        rabbitTemplate.convertAndSend(MqConstant.DELAY_EXCHANGE,
                MqConstant.DELAY_USER_COUPON_ROUTING_KEY, msg, message -> {
                    // 添加延迟消息属性
                    message.getMessageProperties().setDelay(delayValue * 1000);
                    return message;
                });
    }
}
