package com.cultural.heritage.service.user.impl;

import cn.binarywang.wx.miniapp.bean.WxMaJscode2SessionResult;
import cn.hutool.core.util.RandomUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.cultural.heritage.common.ErrorCode;
import com.cultural.heritage.constant.CommonConstant;
import com.cultural.heritage.constant.UserConstant;
import com.cultural.heritage.exception.BusinessException;
import com.cultural.heritage.exception.ThrowUtils;
import com.cultural.heritage.mapper.UserMapper;
import com.cultural.heritage.model.dto.user.UserQueryRequest;
import com.cultural.heritage.model.entity.User;
import com.cultural.heritage.model.enums.UserRoleEnum;
import com.cultural.heritage.model.vo.user.UserVO;
import com.cultural.heritage.service.user.UserService;
import com.cultural.heritage.utils.SqlUtils;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.servlet.http.HttpSession;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.DigestUtils;

import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

import static com.cultural.heritage.constant.UserConstant.SALT;
import static com.cultural.heritage.constant.UserConstant.USER_LOGIN_STATE;

/**
 * @author 陈新知
 * @description 针对表【user(用户)】的数据库操作Service实现
 */

@Slf4j
@Service
public class UserServiceImpl extends ServiceImpl<UserMapper, User> implements UserService {


    /**
     * 用户登录
     */
    @Override
    public UserVO userLogin(String userAccount, String userPassword, HttpServletRequest request) {
        if(userAccount.length() < 4) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR, "账号错误");
        }
        if(userPassword.length() < 4) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR, "密码错误");
        }
        String encryptPassword = DigestUtils.md5DigestAsHex((SALT + userPassword).getBytes());
        QueryWrapper<User> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("userAccount", userAccount);
        queryWrapper.eq("userPassword", encryptPassword);
        User user = this.baseMapper.selectOne(queryWrapper);
        if (user == null) {
            log.info("user login failed, userAccount cannot match userPassword");
            throw new BusinessException(ErrorCode.PARAMS_ERROR, "用户不存在或密码错误");
        }
        HttpSession session = request.getSession();
        System.out.println(session);
        session.setAttribute(USER_LOGIN_STATE, user);
        return this.getUserVO(user);
    }


    @Override
    public UserVO getUserVO(User user) {
        if (user == null) {
            return null;
        }
        UserVO userVO = new UserVO();
        BeanUtils.copyProperties(user, userVO);
        return userVO;
    }


    @Override
    public void validUser(User user, boolean update) {
        String userName = user.getUserName();
        String userPassword = user.getUserPassword();
        String userAvatar = user.getUserAvatar();
        String userRole = user.getUserRole();
        Long id = user.getId();
        ThrowUtils.throwIf(update && id == null, ErrorCode.OPERATION_ERROR, "id字段为null");
        if (StringUtils.isAnyBlank(userName, userAvatar, userRole, userPassword)) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR, "存在参数为空");
        }
    }


    @Override
    public boolean userLogout(HttpServletRequest request) {
        HttpSession session = request.getSession();
        if (session.getAttribute(USER_LOGIN_STATE) == null) {
            throw new BusinessException(ErrorCode.NOT_LOGIN_ERROR);
        }
        session.removeAttribute(USER_LOGIN_STATE);
        return true;
    }

    /**
     * 获取当前登录用户
     */
    @Override
    public User getLoginUser(HttpServletRequest request) {
        HttpSession session = request.getSession();
        Object userObj = session.getAttribute(USER_LOGIN_STATE);
        User currentUser = (User) userObj;
        if (currentUser == null || currentUser.getId() == null) {
            throw new BusinessException(ErrorCode.NOT_LOGIN_ERROR);
        }
        //根据id进行查询
        Long userId = currentUser.getId();
        currentUser = this.getById(userId);
        if (currentUser == null) {
            throw new BusinessException(ErrorCode.NOT_LOGIN_ERROR);
        }
        //被封号
        if (UserRoleEnum.BAN.getValue().equals(currentUser.getUserRole())) {
            throw new BusinessException(ErrorCode.FORBIDDEN_ERROR);
        }
        return currentUser;
    }

    /**
     * 获取用户查询条件
     */
    @Override
    public QueryWrapper<User> getQueryWrapper(UserQueryRequest userQueryRequest) {
        if (userQueryRequest == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR, "请求参数为空");
        }
        Long id = userQueryRequest.getId();
        String miniOpenId = userQueryRequest.getMiniOpenId();
        String userName = userQueryRequest.getUserName();
        String userRole = userQueryRequest.getUserRole();
        String sortField = userQueryRequest.getSortField();
        String sortOrder = userQueryRequest.getSortOrder();

        QueryWrapper<User> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq(id != null, "id", id);
        queryWrapper.eq(StringUtils.isNotBlank(miniOpenId), "miniOpenId", miniOpenId);
        queryWrapper.eq(StringUtils.isNotBlank(userRole), "userRole", userRole);
        queryWrapper.like(StringUtils.isNotBlank(userName), "userName", userName);
        queryWrapper.orderBy(SqlUtils.validSortField(sortField), sortOrder.equals(CommonConstant.SORT_ORDER_ASC),
                sortField);
        return queryWrapper;
    }

    @Override
    public List<UserVO> getUserVO(List<User> userList) {
        if (CollectionUtils.isEmpty(userList)) {
            return new ArrayList<>();
        }
        return userList.stream().map(this::getUserVO).collect(Collectors.toList());
    }

    /**
     * 用户登录（微信小程序）
     */
    @Override
    public UserVO userLoginByMpOpen(WxMaJscode2SessionResult sessionInfo, HttpServletRequest request) {
        String openid = sessionInfo.getOpenid();
        //  单机锁
        synchronized (openid.intern()) {
            // 查询用户是否存在
            QueryWrapper<User> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("miniOpenId", openid);
            User user = this.getOne(queryWrapper);
            // 被封号，禁止登录
            if (user != null && UserRoleEnum.BAN.getValue().equals(user.getUserRole())) {
                throw new BusinessException(ErrorCode.FORBIDDEN_ERROR, "该用户已封禁，禁止登录");
            }
            // 用户不存在则创建
            if (user == null) {
                user = new User();
                String userAccount = RandomUtil.randomNumbers(14);
                String userPassword = RandomUtil.randomString(16);
                user.setUserAccount(userAccount);
                user.setUserPassword(userPassword);
                user.setMiniOpenId(openid);
                user.setUserName("非遗雅士");
                user.setUserRole("user");
                user.setPoints(10000);
                user.setUserAvatar(UserConstant.USER_DEFAULT_AVATAR);
                boolean result = this.save(user);
                ThrowUtils.throwIf(!result, ErrorCode.SYSTEM_ERROR, "登录失败");
                // 记住用户的登录态
            }
            request.getSession().setAttribute(USER_LOGIN_STATE, user);
            return this.getUserVO(user);
        }
    }


    /**
     * 用户添加（扣除）积分
     */
    @Override
    public void addOrSubUserPoints(Long userId, boolean isAdd, Integer points) {
        User user = this.getById(userId);
        if (isAdd) {
            user.setPoints(user.getPoints() + points);
        } else {
            user.setPoints(Math.max(user.getPoints() - points, 0));
        }
        boolean update = this.updateById(user);
        ThrowUtils.throwIf(!update, ErrorCode.OPERATION_ERROR, "积分更新失败");
    }

}
